import axios from 'axios';
import knex from 'knex';
import fs from 'fs';
import { google } from 'googleapis';
/**
 * @type Warehouse
 * Type contains information about a warehouse.
 * I dunno what other things aside from name means,
 * anyways it works and I don't think I need to know :P
 */
type Warehouse = {
    name: string,
    box_delivery_and_storage_expr: Number,
    box_delivery_base: Number,
    box_delivery_liter: Number,
    box_storage_base: Number,
    box_storage_liter: Number
};

const dbClient = knex({
    client: "pg",
    connection: {
        host: process.env.DB_HOST,
        port: Number.parseInt(process.env.DB_PORT!),
        user: process.env.DB_USER,
        password: process.env.DB_PASS,
        database: process.env.DB_NAME
    }
})
/**
 * This function executes file ./db_schema.psql that contains the DB schema in format "create if not exists"
 */
const initDb = async () => {
    console.log("Starting database initialization.");
    const initializeQuery = fs.readFileSync('./db_schema.sql').toString();

    await dbClient.raw(initializeQuery);
    console.log("Database initialization ended.");
}

/**
 * This function makes request to the WB API
 * and puts the data to the DB.
 */
const request = async () => {
    console.log("starting request")
    const data = {
        response: {
            data: {
                dtNextBox: "2024-02-01",
                dtTillMax: "2024-03-31",
                warehouseList: [
                    {
                        boxDeliveryAndStorageExpr: "160",
                        boxDeliveryBase: "48",
                        boxDeliveryLiter: "11,2",
                        boxStorageBase: "0,1",
                        boxStorageLiter: "0,1",
                        warehouseName: "Коледино"
                    }
                ]
            }
        }
    }
    /*
    const data = await axios.get(`https://common-api.wildberries.ru/api/v1/tariffs/box`, {
        params: {
            date: new Date().toISOString().split('T')[0]
        },
        headers: {
            "Authorization": `Bearer ${process.env.WB_TOKEN}`
        }
    });
    */
   const warehouses: Warehouse[] = []
    data.response.data.warehouseList.forEach(warehouse => {
        const warehoseData: Warehouse = {
            name: warehouse.warehouseName,
            box_delivery_and_storage_expr: Number.parseInt(warehouse.boxDeliveryAndStorageExpr),
            box_delivery_base: Number.parseInt(warehouse.boxDeliveryBase),
            box_delivery_liter: Number.parseInt(warehouse.boxDeliveryLiter),
            box_storage_base: Number.parseFloat(warehouse.boxStorageBase.replace(",", ".")),
            box_storage_liter: Number.parseFloat(warehouse.boxStorageLiter.replace(",", "."))
        };
        warehouses.push(warehoseData);
        dbClient("warehouses")
            .insert(warehoseData)
            .onConflict("name")
            .merge()
            .catch(e => console.log(e));
    });
    return warehouses;
}

/**
 * This function handles data export to the google sheets.
 */
const exportData = async (warehouses: Warehouse[]) => {
    const auth = new google.auth.GoogleAuth({
        keyFile: "./service-account-file.json",
        scopes: ["https://www.googleapis.com/auth/spreadsheets"]
    });

    const sheets = google.sheets({
        version: "v4",
        auth
    });

    const spreadsheetsIds = process.env.SPREADSHEETS_IDS?.split(':')

    warehouses.forEach( async (warehouse, index) => {

        const headerValues = [
            ['Склад', 'box delivery and storage expr', 'box delivery base', 'box delivery liter', 'box storage_base', 'box storage liter']
        ];

        spreadsheetsIds?.forEach(async spreadsheetId => {
            // sheets.spreadsheets.values.clear();
            const alphabet = "ABCDEFGHIJKLMNOPQRSTUVWXYZ"

            // Print header.
            await sheets.spreadsheets.values.update({
                spreadsheetId,
                range: `${process.env.SPREADSHEETS_NAME}!${alphabet[0]}1:${alphabet[headerValues[0].length-1]}1`,
                valueInputOption: "RAW",
                resource: {
                    values: headerValues
                }
            })

    
            const values = [
                [warehouse.name, warehouse.box_delivery_and_storage_expr, warehouse.box_delivery_base, warehouse.box_delivery_liter, warehouse.box_storage_base, warehouse.box_storage_liter]
            ];
        
        
            const reponse = await sheets.spreadsheets.values.update({
                spreadsheetId,
                range: `${process.env.SPREADSHEETS_NAME}!${alphabet[index]}${index+2}:${alphabet[index+values[index].length]}${index+2}`,
                valueInputOption: "RAW",
                resource: {
                    values
                }
            })
        })
    })
    
}

/**
 * This function call DB initialization and sets timeout (whit it reads from .env file) for the request() function
 */
const start = async () => {
    await initDb();

    const warehouses = await request();
    exportData(warehouses);

    setInterval(async () => {
        const warehouses = await request();
        exportData(warehouses);
    }, Number.parseInt(process.env.TIME_BETWEEN_REQUESTS!) * 1000);
}

await start();